import networkx as nx

import pytest
from evrouting.gasstation.routing import contract_graph, dijkstra
from evrouting.graph_tools import label, CONSUMPTION_KEY
from tests.config import edge_case, get_graph, gasstation, init_config


class TestDjikstra:
    @pytest.mark.parametrize("u,v,min_len", [(0, 1, 1), (0, 2, 3), (0, 4, 2), (0, 6, 3)])
    def test_djikstra(self, u, v, min_len):
        conf: dict = init_config(gasstation)
        assert dijkstra(conf['G'], u, v, CONSUMPTION_KEY) == min_len

    def test_djikstra_via_node(self):
        conf: dict = init_config(edge_case)

        assert dijkstra(conf['G'], 0, 2, CONSUMPTION_KEY) == 2


class TestContraction:
    def test_contraction(self):
        conf: dict = init_config(gasstation)
        G: nx.Graph = conf['G']
        H: nx.Graph = contract_graph(G, conf['charging_stations'], conf['U'])

        # Check available gas stations
        nodes = set(label(H, n) for n in H.nodes)
        assert nodes == {'s', 'a', 'c'}


class TestRouting:
    def test_shortest_path(self):
        G = get_graph(edge_case)

        path = shortest_path(
            G,
            s=edge_case['s'],
            t=edge_case['t'],
            b_0=edge_case['b_0'],
            b_t=edge_case['b_t'],
            U=edge_case['U']
        )

        assert path == [(0, 4), (2, 0)]
